MODULE LCDEpson;
(* =========================================================================  
   Example ARM Oberon Module  
 
   Description:
     Epson S1D15G00 LCD Controller Driver for 132x132 LCD Nokia-type display

   Target:
     LPC17xx systems
     
   Tested on: 
     Olimex LPC1766-STK Development Prototype Board
   
   Reference:
     Seiko Epson S1D15G00 Series DataSheet Rev 1.0 2001.
     
   (c) 2009-2012 CFB Software
   http://www.astrobe.com
   
   ========================================================================= *)
IMPORT Timer, MCU, ResData, SPI, SYSTEM;

CONST 
  DISON   = 0AFH; (* Display on *)
  DISINV  = 0A7H; (* Inverse display *)
  COMSCN  = 0BBH; (* Common scan direction *)
  DISCTL  = 0CAH; (* Display control *)
  SLPOUT  = 094H; (* Sleep out *)
  PASET   = 075H; (* Page address set *)
  CASET   = 015H; (* Column address set *)
  DATCTL  = 0BCH; (* Data scan direction, etc. *)
  RAMWR   = 05CH; (* Writing to memory *)
  OSCON   = 0D1H; (* Internal oscillation on *)
  PWRCTR  = 020H; (* Power control *)
  VOLCTR  = 081H; (* Electronic volume control *)

  (* 12-bit color definitions *)
  White*   = 0FFFH;
  Black*   = 0000H;
  Red*     = 0F00H;
  Green*   = 00F0H;
  Blue*    = 000FH;
  Cyan*    = 00FFH;
  Magenta* = 0F0FH;
  Yellow*  = 0FF0H;
  Brown*   = 0B22H;
  Orange*  = 0FA0H;
  Pink*    = 0F6AH;
  
  MaxX* = 131;
  MaxY* = 131;
  
  (* 8 x 10 font *)
  MaxCol* = MaxX DIV 8;
  MaxRow* = MaxY DIV 10;

VAR
  font: ResData.Resource;
    
  (* ========================================================================= *) 
  PROCEDURE LoadFont*(name: ARRAY OF CHAR): BOOLEAN;
  BEGIN
    ResData.Open(font, name);
    RETURN font # NIL 
  END LoadFont;
    
  (* ========================================================================= *) 
  PROCEDURE CharToFont(ch: CHAR; VAR fontCh1, fontCh2: INTEGER);
  VAR
    index: INTEGER;
  BEGIN
    IF ORD(ch) < ORD(" ") THEN ch := "." END;
    index := (ORD(ch) - ORD(" ")) * 2;
    ResData.GetInt(font, index, fontCh1);
    ResData.GetInt(font, index + 1, fontCh2)
  END CharToFont;
  
  (* ========================================================================= *) 
  PROCEDURE SendData(data: INTEGER);
  BEGIN
    SPI.SendData(ORD(BITS(data) + {8}))
  END SendData;
  
  (* ========================================================================= *) 
  PROCEDURE SendCommand(data: INTEGER);
  BEGIN
    SPI.SendData(data)
  END SendCommand;
  
 (* ************************************************************************************* *)
 (* Draws two dots in the specified color at the specified x and y coordinates *)
 (* color = 12-bit color value xxxxrrrrggggbbbb *)
 (* 0000 colour is off, 1111 colour is full on *)
  PROCEDURE DrawDot*(colour, x, y: INTEGER);
  BEGIN
    IF (x <= MaxX) & (y <= MaxY) & (x >= 0) & (y >= 0) THEN 
      (* Column address *)
      SendCommand(CASET); 
      SendData(x); 
      SendData(x); 
      (* Row address *)
      SendCommand(PASET); 
      SendData(y); 
      SendData(y); 
      (* Draw the dot twice = 2 x 1.5 bytes *)
      SendCommand(RAMWR); 
      SendData(LSR(colour, 4));
      SendData(LSL(colour MOD 10H, 4) + LSR(colour, 8) MOD 10H);
      SendData(colour MOD 100H)
    END 
  END DrawDot;


  (* ========================================================================= *) 
  PROCEDURE FillRectangle*(colour, x1, y1, x2, y2: INTEGER);
  VAR
    i: INTEGER;
  BEGIN
    ASSERT((x1 >= 0) & (y1 >= 0), 100);
    ASSERT((x2 >= x1) & (y2 >= y1), 101);    
    ASSERT((x2 <= MaxX) & (y2 <= MaxY), 102);
    (* Column address *)
    SendCommand(CASET); 
    SendData(x1); 
    SendData(x2); 
    (* Row address *)
    SendCommand(PASET); 
    SendData(y1); 
    SendData(y2); 
    (* Now write the pixels *)
    SendCommand(RAMWR); 
    FOR i := 0 TO ((x2 - x1 + 1) * (y2 - y1 + 1)) DIV 2 - 1 DO
      SendData(LSR(colour, 4));
      SendData(LSL(colour MOD 10H, 4) + LSR(colour, 8) MOD 10H);
      SendData(colour MOD 100H)
    END
  END FillRectangle;


  (* ========================================================================= *) 
  PROCEDURE DrawChar*(colour: INTEGER; ch: CHAR; x1, y1: INTEGER);
  VAR
    i: INTEGER;
    colour1, colour2: INTEGER;
    x2, y2: INTEGER;
    font, font1, font2: INTEGER;
  BEGIN
    CharToFont(ch, font1, font2);
    x2 := x1 + 5;
    y2 := y1 + 7;
    ASSERT((x1 >= 0) & (y1 >= 0), 100);
    ASSERT((x2 >= x1) & (y2 >= y1), 101);    
    ASSERT((x2 <= MaxX) & (y2 <= MaxY), 102);
    (* Column address *)
    SendCommand(CASET); 
    SendData(x1); 
    SendData(x2); 
    (* Row address *)
    SendCommand(PASET); 
    SendData(y1); 
    SendData(y2); 
    (* Now write the pixels *)
    SendCommand(RAMWR); 
    font := font1;
    FOR i := 0 TO ((x2 - x1 + 1) * (y2 - y1 + 1)) DIV 2 - 1 DO
      IF i = 16 THEN font := LSL(font2, 16) END;
      IF font < 0 THEN colour1 := colour ELSE colour1 := Black END;
      font := LSL(font, 1);
      IF font < 0 THEN colour2 := colour ELSE colour2 := Black END;
      font := LSL(font, 1);
      SendData(LSR(colour1, 4));
      SendData(LSL(colour1 MOD 10H, 4) + LSR(colour2, 8) MOD 10H);
      SendData(colour2 MOD 100H)
    END
  END DrawChar;


  (* ========================================================================= *) 
  PROCEDURE ClearScreen*(Colour: INTEGER);
  BEGIN
    FillRectangle(Colour, 0, 0, MaxX, MaxY)
  END ClearScreen;
  
  
  (* ========================================================================= *) 
  (* Initialise the Epson S1D15G00 LCD Controller *)
  PROCEDURE Init*;
  CONST
    SPIBus = 0;
    nBits = 9;
    useSSEL = TRUE;
  VAR
    s: SET;
  BEGIN
    SPI.Init(SPIBus, nBits, useSSEL);
    
    (*  LCD reset P3.25, Backlight P3.26 *) 
    SYSTEM.GET(MCU.FIO3DIR, s); 
    SYSTEM.PUT(MCU.FIO3DIR, s + {25, 26});
  
    (* Hardware reset and switch on backlight*)
    SYSTEM.PUT(MCU.FIO3CLR, {25}); 
    SYSTEM.PUT(MCU.FIO3SET, {25, 26}); 
  
    (* Display control *)
    SendCommand(DISCTL); 
    SendData(000H); (* P1: 0x00 = 2 divisions, switching period=8(default) *)
    SendData(020H); (* P2: 0x20 = nlines/4 - 1 = 132/4 - 1 = 32) *)
    SendData(000H); (* P3: 0x00 = no inversely highlighted lines *)
    (* COM scan *)
    SendCommand(COMSCN); 
    SendData(1); (* P1: 0x01 = Scan 1->80, 160<-81 *)
    (* Internal oscilator ON *)
    SendCommand(OSCON); 
    (* Sleep out *)
    SendCommand(SLPOUT); 
    (* Power control *)
    SendCommand(PWRCTR); 
    SendData(00FH); (* reference voltage regulator on, circuit voltage follower on, BOOST ON *)
    (* Inverse display *)
    SendCommand(DISINV);
    (* Data control *)
    SendCommand(DATCTL); 
    SendData(004H); (* P1: 0x04 = Display: Normal, Rotation: Normal, Address-scan: page direction *)
    SendData(000H); (* P2: 0x00 = RGB sequence(default value) *)
    SendData(002H); (* P3: 0x02 = 16 gray-scale *)
    (* Voltage control(contrast setting) *)
    SendCommand(VOLCTR); 
    SendData(32); (* P1 = 32 volume value(experiment with this value to get the best contrast) 0..63 *)
    SendData(3);  (* P2 = 3 resistance ratio(only value that works) *)
    (* allow power supply to stabilize *)
    Timer.MSecDelay(100); 
    (* turn on the display *)
    ClearScreen(Black);
    SendCommand(DISON); 
  END Init;
  
END LCDEpson.
